@@ -21,6 +21,9 @@
 #include <exec/types.h>
 #include <proto/graphics.h>
 #include <intuition/classusr.h>
+#include <libraries/Picasso96.h>
+
+#define AMI_BITMAP_FORMAT RGBFB_R8G8B8A8
 
 struct bitmap {
 	int width;
