@@ -16,36 +16,44 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "desktop/browser.h"
-#include "amiga/menu.h"
-#include "utils/messages.h"
-#include "amiga/utf8.h"
-#include <libraries/gadtools.h>
-#include <proto/asl.h>
-#include "amiga/options.h"
-#include "desktop/gui.h"
-#include "amiga/hotlist.h"
+#include <string.h>
+
 #include <proto/dos.h>
+#include <proto/asl.h>
+#include <proto/exec.h>
+#include <proto/intuition.h>
+#include <dos/anchorpath.h>
+#include <libraries/gadtools.h>
+
+#include "amiga/menu.h"
+#include "amiga/utf8.h"
+#include "amiga/options.h"
+#include "amiga/hotlist.h"
 #include "amiga/gui.h"
 #include "amiga/save_pdf.h"
-#include "desktop/save_text.h"
-#include "desktop/save_pdf/pdf_plotters.h"
-#include <string.h>
 #include "amiga/tree.h"
 #include "amiga/history.h"
 #include "amiga/cookies.h"
-#include <proto/exec.h>
 #include "amiga/arexx.h"
-#include "amiga/save_complete.h"
-#include "utils/url.h"
-#include <dos/anchorpath.h>
-#include "desktop/textinput.h"
 #include "amiga/search.h"
 #include "amiga/history_local.h"
 #include "amiga/bitmap.h"
 #include "amiga/iff_dr2d.h"
 #include "amiga/clipboard.h"
+#include "amiga/gui_options.h"
+#include "amiga/print.h"
+#include "amiga/download.h"
+
 #include "content/fetch.h"
+#include "desktop/browser.h"
+#include "desktop/gui.h"
+#include "desktop/save_text.h"
+#include "desktop/save_pdf/pdf_plotters.h"
+#include "desktop/save_complete.h"
+#include "desktop/selection.h"
+#include "desktop/textinput.h"
+#include "utils/messages.h"
+#include "utils/url.h"
 
 BOOL menualreadyinit;
 const char * const netsurf_version;
@@ -67,6 +75,11 @@ void ami_free_menulabs(void)
 	{
 		if(menulab[i] && (menulab[i] != NM_BARLABEL)) ami_utf8_free(menulab[i]);
 	}
+
+	for(i=AMI_MENU_MAX+1;i<=AMI_MENU_AREXX_MAX;i++)
+	{
+		if(menulab[i] && (menulab[i] != NM_BARLABEL)) free(menulab[i]);
+	}
 }
 
 void ami_init_menulabs(void)
@@ -76,7 +89,7 @@ void ami_init_menulabs(void)
 	menulab[2] = ami_utf8_easy((char *)messages_get("NewTab"));
 	menulab[3] = NM_BARLABEL;
 	menulab[4] = ami_utf8_easy((char *)messages_get("OpenFile"));
-	menulab[5] = ami_utf8_easy((char *)messages_get("SaveAs"));
+	menulab[5] = ami_utf8_easy((char *)messages_get("SaveAsNS"));
 	menulab[6] = ami_utf8_easy((char *)messages_get("Source"));
 	menulab[7] = ami_utf8_easy((char *)messages_get("TextNS"));
 	menulab[8] = ami_utf8_easy((char *)messages_get("SaveCompNS"));
@@ -86,32 +99,43 @@ void ami_init_menulabs(void)
 	menulab[12] = ami_utf8_easy((char *)messages_get("CloseTab"));
 	menulab[13] = ami_utf8_easy((char *)messages_get("CloseWindow"));
 	menulab[14] = NM_BARLABEL;
-	menulab[15] = ami_utf8_easy((char *)messages_get("About"));
-	menulab[16] = ami_utf8_easy((char *)messages_get("Quit"));
-	menulab[17] = ami_utf8_easy((char *)messages_get("Edit"));
-	menulab[18] = ami_utf8_easy((char *)messages_get("CopyNS"));
-	menulab[19] = ami_utf8_easy((char *)messages_get("PasteNS"));
-	menulab[20] = ami_utf8_easy((char *)messages_get("SelectAllNS"));
-	menulab[21] = ami_utf8_easy((char *)messages_get("ClearNS"));
-	menulab[22] = ami_utf8_easy((char *)messages_get("Browser"));
-	menulab[23] = ami_utf8_easy((char *)messages_get("FindTextNS"));
-	menulab[24] = NM_BARLABEL;
-	menulab[25] = ami_utf8_easy((char *)messages_get("normal"));
-	menulab[26] = ami_utf8_easy((char *)messages_get("HistLocalNS"));
-	menulab[27] = ami_utf8_easy((char *)messages_get("HistGlobalNS"));
+	menulab[15] = ami_utf8_easy((char *)messages_get("PrintNS"));
+	menulab[16] = NM_BARLABEL;
+	menulab[17] = ami_utf8_easy((char *)messages_get("About"));
+	menulab[18] = ami_utf8_easy((char *)messages_get("Quit"));
+	menulab[19] = ami_utf8_easy((char *)messages_get("Edit"));
+	menulab[20] = ami_utf8_easy((char *)messages_get("CutNS"));
+	menulab[21] = ami_utf8_easy((char *)messages_get("CopyNS"));
+	menulab[22] = ami_utf8_easy((char *)messages_get("PasteNS"));
+	menulab[23] = NM_BARLABEL;
+	menulab[24] = ami_utf8_easy((char *)messages_get("SelectAllNS"));
+	menulab[25] = ami_utf8_easy((char *)messages_get("ClearNS"));
+	menulab[26] = ami_utf8_easy((char *)messages_get("Browser"));
+	menulab[27] = ami_utf8_easy((char *)messages_get("FindTextNS"));
 	menulab[28] = NM_BARLABEL;
-	menulab[29] = ami_utf8_easy((char *)messages_get("ShowCookies"));
-	menulab[30] = ami_utf8_easy((char *)messages_get("Hotlist"));
-	menulab[31] = ami_utf8_easy((char *)messages_get("HotlistAdd"));
-	menulab[32] = ami_utf8_easy((char *)messages_get("HotlistShowNS"));
+	menulab[29] = ami_utf8_easy((char *)messages_get("HistLocalNS"));
+	menulab[30] = ami_utf8_easy((char *)messages_get("HistGlobalNS"));
+	menulab[31] = NM_BARLABEL;
+	menulab[32] = ami_utf8_easy((char *)messages_get("ShowCookies"));
 	menulab[33] = NM_BARLABEL;
+	menulab[34] = ami_utf8_easy((char *)messages_get("ScaleNS"));
+	menulab[35] = ami_utf8_easy((char *)messages_get("ScaleDec"));
+	menulab[36] = ami_utf8_easy((char *)messages_get("ScaleNorm"));
+	menulab[37] = ami_utf8_easy((char *)messages_get("ScaleInc"));
+	menulab[38] = ami_utf8_easy((char *)messages_get("Redraw"));
+	menulab[39] = ami_utf8_easy((char *)messages_get("Hotlist"));
+	menulab[40] = ami_utf8_easy((char *)messages_get("HotlistAdd"));
+	menulab[41] = ami_utf8_easy((char *)messages_get("HotlistShowNS"));
+	menulab[42] = NM_BARLABEL;
 
 	menulab[AMI_MENU_HOTLIST_MAX] = ami_utf8_easy((char *)messages_get("Settings"));
-	menulab[AMI_MENU_HOTLIST_MAX+1] = ami_utf8_easy((char *)messages_get("SnapshotWindow"));
-	menulab[AMI_MENU_HOTLIST_MAX+2] = ami_utf8_easy((char *)messages_get("SettingsSave"));
-	menulab[AMI_MENU_HOTLIST_MAX+3] = ami_utf8_easy((char *)messages_get("ARexx"));
-	menulab[AMI_MENU_HOTLIST_MAX+4] = ami_utf8_easy((char *)messages_get("ARexxExecute"));
-	menulab[AMI_MENU_HOTLIST_MAX+5] = NM_BARLABEL;
+	menulab[AMI_MENU_HOTLIST_MAX+1] = ami_utf8_easy((char *)messages_get("SettingsEdit"));
+	menulab[AMI_MENU_HOTLIST_MAX+2] = NM_BARLABEL;
+	menulab[AMI_MENU_HOTLIST_MAX+3] = ami_utf8_easy((char *)messages_get("SnapshotWindow"));
+	menulab[AMI_MENU_HOTLIST_MAX+4] = ami_utf8_easy((char *)messages_get("SettingsSave"));
+	menulab[AMI_MENU_HOTLIST_MAX+5] = ami_utf8_easy((char *)messages_get("ARexx"));
+	menulab[AMI_MENU_HOTLIST_MAX+6] = ami_utf8_easy((char *)messages_get("ARexxExecute"));
+	menulab[AMI_MENU_HOTLIST_MAX+7] = NM_BARLABEL;
 }
 
 struct NewMenu *ami_create_menu(ULONG type)
@@ -134,21 +158,30 @@ struct NewMenu *ami_create_menu(ULONG type)
 			  	{ NM_ITEM,0,"K",0,0,0,}, // close tab
 			  	{ NM_ITEM,0,0,0,0,0,}, // close window
 			  	{ NM_ITEM,NM_BARLABEL,0,0,0,0,},
+			  	{ NM_ITEM,0,"P",0,0,0,}, // print
+			  	{ NM_ITEM,NM_BARLABEL,0,0,0,0,},
 			  	{ NM_ITEM,0,"?",0,0,0,}, // about
 			  	{ NM_ITEM,0,"Q",0,0,0,}, // quit
 			  	{NM_TITLE,0,0,0,0,0,}, // edit
+			  	{ NM_ITEM,0,"X",0,0,0,}, // cut
 			  	{ NM_ITEM,0,"C",0,0,0,}, // copy
 			  	{ NM_ITEM,0,"V",0,0,0,}, // paste
+			  	{ NM_ITEM,NM_BARLABEL,0,0,0,0,},
 			  	{ NM_ITEM,0,"A",0,0,0,}, // select all
 			  	{ NM_ITEM,0,"Z",0,0,0,}, // clear selection
 			  	{NM_TITLE,0,0,0,0,0,}, // browser
 			  	{ NM_ITEM,0,"F",0,0,0,}, // find in page
 			  	{ NM_ITEM,NM_BARLABEL,0,0,0,0,},
-			  	{NM_IGNORE,0,0,0,0,0,}, // was test option for scaling
 			  	{ NM_ITEM,0,0,0,0,0,}, // local history
 			  	{ NM_ITEM,0,0,0,0,0,}, // global history
 			  	{ NM_ITEM,NM_BARLABEL,0,0,0,0,},
 			  	{ NM_ITEM,0,0,0,0,0,}, // cookies
+			  	{ NM_ITEM,NM_BARLABEL,0,0,0,0,},
+			  	{ NM_ITEM,0,0,0,0,0,}, // scale
+			  	{  NM_SUB,0,"-",0,0,0,}, // decrease
+			  	{  NM_SUB,0,"=",0,0,0,}, // normal
+			  	{  NM_SUB,0,"+",0,0,0,}, // increase
+			  	{ NM_ITEM,0,0,0,0,0,}, // redraw
 				{NM_TITLE,0,0,0,0,0,}, // hotlist
 				{ NM_ITEM,0,0,0,0,0,}, // add to hotlist
 			  	{ NM_ITEM,0,"H",0,0,0,}, // show hotlist (treeview)
@@ -194,6 +227,8 @@ struct NewMenu *ami_create_menu(ULONG type)
 				{ NM_IGNORE,0,0,0,0,0,}, // ** hotlist entry **
 				{ NM_IGNORE,0,0,0,0,0,}, // ** hotlist entry **
 				{NM_TITLE,0,0,0,0,0,}, // settings
+			  	{ NM_ITEM,0,0,0,0,0,}, // edit prefs
+			  	{ NM_ITEM,NM_BARLABEL,0,0,0,0,},
 				{ NM_ITEM,0,0,0,0,0,}, // snapshot window
 				{ NM_ITEM,0,0,0,0,0,}, // save settings
 				{NM_TITLE,0,0,0,0,0,}, // arexx
@@ -234,11 +269,11 @@ struct NewMenu *ami_create_menu(ULONG type)
 
 	menu[1].nm_Flags = menuflags;
 	menu[2].nm_Flags = menuflags;
-	menu[9].nm_Flags = menuflags;
-	menu[10].nm_Flags = menuflags;
+	menu[12].nm_Flags = menuflags;
+	menu[13].nm_Flags = menuflags;
 
 #ifndef WITH_PDF_EXPORT
-	menu[7].nm_Flags = NM_ITEMDISABLED;
+	menu[9].nm_Flags = NM_ITEMDISABLED;
 #endif
 
 	if(!menualreadyinit)
@@ -288,6 +323,9 @@ void ami_menu_arexx_scan(struct NewMenu *menu)
 					for(ead = (struct ExAllData *)buffer; ead; ead = ead->ed_Next)
 					{
 						if(item >= AMI_MENU_AREXX_MAX) continue;
+						if(!strcasecmp(ead->ed_Name, option_arexx_startup)) continue;
+						if(!strcasecmp(ead->ed_Name, option_arexx_shutdown)) continue;
+
 						if(EAD_IS_FILE(ead))
 						{
 							menu[item].nm_Type = NM_ITEM;
@@ -395,6 +433,9 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 	bool opentab=true;
 	char *temp;
 	BPTR lock = 0;
+	char *source_data;
+	ULONG source_size;
+	struct bitmap *bm;
 
 	tgw.tab_node = NULL;
 	tgw.tab = 0;
@@ -406,11 +447,11 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 			switch(itemnum)
 			{
 				case 0: // new window
-					bw = browser_window_create(gwin->bw->current_content->url,gwin->bw, 0, true, openwin);
+					bw = browser_window_create(option_homepage_url, gwin->bw, 0, true, openwin);
 				break;
 
 				case 1: // new tab
-					bw = browser_window_create(gwin->bw->current_content->url,gwin->bw, 0, true, opentab);
+					bw = browser_window_create(NULL, gwin->bw, 0, true, opentab);
 				break;
 
 				case 3: // open local file
@@ -447,7 +488,7 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 							if(AslRequestTags(savereq,
 								ASLFR_TitleText,messages_get("NetSurf"),
 								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(gwin->bw->current_content->url),
+								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
 								strlcpy(&fname,savereq->fr_Drawer,1024);
@@ -455,9 +496,10 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
 								if(fh = FOpen(fname,MODE_NEWFILE,0))
 								{
-									FWrite(fh,gwin->bw->current_content->source_data,1,gwin->bw->current_content->source_size);
+									if((source_data = content_get_source_data(gwin->bw->current_content, &source_size)))
+										FWrite(fh,source_data, 1, source_size);
 									FClose(fh);
-									SetComment(fname,gwin->bw->current_content->url);
+									SetComment(fname, content_get_url(gwin->bw->current_content));
 								}
 								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
 							}
@@ -467,14 +509,14 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 							if(AslRequestTags(savereq,
 								ASLFR_TitleText,messages_get("NetSurf"),
 								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(gwin->bw->current_content->url),
+								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
 								strlcpy(&fname,savereq->fr_Drawer,1024);
 								AddPart(fname,savereq->fr_File,1024);
 								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
 								save_as_text(gwin->bw->current_content,fname);
-								SetComment(fname,gwin->bw->current_content->url);
+								SetComment(fname,content_get_url(gwin->bw->current_content));
 								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
 							}
 						break;
@@ -483,7 +525,7 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 							if(AslRequestTags(savereq,
 								ASLFR_TitleText,messages_get("NetSurf"),
 								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(gwin->bw->current_content->url),
+								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
 								strlcpy(&fname,savereq->fr_Drawer,1024);
@@ -493,7 +535,9 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 								{
 									UnLock(lock);
 									save_complete(gwin->bw->current_content,fname);
-									SetComment(fname,gwin->bw->current_content->url);
+									SetComment(fname,content_get_url(gwin->bw->current_content));
+									ami_superimpose_favicon(fname,
+										gwin->bw->window->favicon, NULL);
 								}
 								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
 							}
@@ -504,14 +548,16 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 							if(AslRequestTags(savereq,
 								ASLFR_TitleText,messages_get("NetSurf"),
 								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(gwin->bw->current_content->url),
+								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
 								strlcpy(&fname,savereq->fr_Drawer,1024);
 								AddPart(fname,savereq->fr_File,1024);
 								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
 								save_as_pdf(gwin->bw->current_content,fname);
-								SetComment(fname,gwin->bw->current_content->url);
+								SetComment(fname, content_get_url(gwin->bw->current_content));
+								ami_superimpose_favicon(fname,
+									gwin->bw->window->favicon, "pdf");
 								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
 							}
 #endif
@@ -521,25 +567,25 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 							if(AslRequestTags(savereq,
 								ASLFR_TitleText,messages_get("NetSurf"),
 								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(gwin->bw->current_content->url),
+								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
 								TAG_DONE))
 							{
 								strlcpy(&fname,savereq->fr_Drawer,1024);
 								AddPart(fname,savereq->fr_File,1024);
 								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
-								if(gwin->bw->current_content->bitmap)
+								if((bm = content_get_bitmap(gwin->bw->current_content)))
 								{
-									gwin->bw->current_content->bitmap->url = gwin->bw->current_content->url;
-									gwin->bw->current_content->bitmap->title = gwin->bw->current_content->title;
-									bitmap_save(gwin->bw->current_content->bitmap,fname,0);
+									bm->url = content_get_url(gwin->bw->current_content);
+									bm->title = content_get_title(gwin->bw->current_content);
+									bitmap_save(bm, fname, 0);
 								}
 #ifdef WITH_NS_SVG
-								else if(gwin->bw->current_content->type == CONTENT_SVG)
+								else if(content_get_type(gwin->bw->current_content) == CONTENT_SVG)
 								{
 									ami_save_svg(gwin->bw->current_content,fname);
 								}
 #endif
-								SetComment(fname,gwin->bw->current_content->url);
+								SetComment(fname, content_get_url(gwin->bw->current_content));
 								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
 							}
 						break;
@@ -554,7 +600,13 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 					ami_close_all_tabs(gwin);
 				break;
 
-				case 9: // about
+				case 9: // print
+					ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
+					ami_print_ui(gwin->bw->current_content);
+					ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
+				break;
+
+				case 11: // about
 					ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
 
 					TimedDosRequesterTags(
@@ -562,19 +614,29 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 						TDR_TitleString,messages_get("NetSurf"),
 						TDR_Window,gwin->win,
 						TDR_GadgetString,messages_get("OK"),
+#ifndef NDEBUG
+						TDR_FormatString,"NetSurf %s\n%s\n%s (%s)\n\nhttp://www.netsurf-browser.org",
+#else
 						TDR_FormatString,"NetSurf %s\n%s\n\nhttp://www.netsurf-browser.org",
+#endif
 						TDR_Arg1,netsurf_version,
 #ifdef NS_AMIGA_CAIRO
-						TDR_Arg2,"Cairo",
+#ifdef NS_AMIGA_CAIRO_ALL
+						TDR_Arg2,"Cairo (OS4.1+) SObjs build",
 #else
-						TDR_Arg2,"graphics.library",
+						TDR_Arg2,"Part-Cairo (OS4.1+) SObjs build",
 #endif
+#else
+						TDR_Arg2,"graphics.library static build",
+#endif
+						TDR_Arg3,versvn,
+						TDR_Arg4,verdate,
 						TAG_DONE);
 
 					ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
 				break;
 
-				case 10: // quit
+				case 12: // quit
 					ami_quit_netsurf();
 				break;
 			}
@@ -583,36 +645,41 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 		case 1:  // edit
 			switch(itemnum)
 			{
-				case 0: // copy
-					if(gwin->bw->current_content->type <= CONTENT_CSS)
+				case 0: // cut
+					browser_window_key_press(gwin->bw, KEY_CUT_SELECTION);
+				break;
+
+				case 1: // copy
+					if(content_get_type(gwin->bw->current_content) <= CONTENT_CSS)
 					{
 						browser_window_key_press(gwin->bw, KEY_COPY_SELECTION);
-						browser_window_key_press(gwin->bw, KEY_ESCAPE);
+						browser_window_key_press(gwin->bw, KEY_CLEAR_SELECTION);
 					}
-					else if(gwin->bw->current_content->bitmap)
+					else if(bm = content_get_bitmap(gwin->bw->current_content))
 					{
-						gwin->bw->current_content->bitmap->url = gwin->bw->current_content->url;
-						gwin->bw->current_content->bitmap->title = gwin->bw->current_content->title;
-						ami_easy_clipboard_bitmap(gwin->bw->current_content->bitmap);
+						bm->url = content_get_url(gwin->bw->current_content);
+						bm->title = content_get_title(gwin->bw->current_content);
+						ami_easy_clipboard_bitmap(bm);
 					}
 #ifdef WITH_NS_SVG
-					else if(gwin->bw->current_content->type == CONTENT_SVG)
+					else if(content_get_type(gwin->bw->current_content) == CONTENT_SVG)
 					{
 						ami_easy_clipboard_svg(gwin->bw->current_content);
 					}
 #endif
 				break;
 
-				case 1: // paste
+				case 2: // paste
 					browser_window_key_press(gwin->bw, KEY_PASTE);
 					//gui_paste_from_clipboard(&tgw,0,0);
 				break;
 
-				case 2: // select all
+				case 4: // select all
 					browser_window_key_press(gwin->bw, KEY_SELECT_ALL);
+					gui_start_selection(gwin->bw->window);
 				break;
 
-				case 3: // clear selection
+				case 5: // clear selection
 					browser_window_key_press(gwin->bw, KEY_CLEAR_SELECTION);
 				break;
 			}
@@ -625,19 +692,6 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 					ami_search_open(gwin->bw->window);
 				break;
 
-				case 1: // size
-					switch(subnum)
-					{
-						case 0: // normal
-							gwin->bw->scale = 1.0;
-						break;
-
-						case 1: // double
-							gwin->bw->scale = 2.0;
-						break;
-					}
-				break;
-
 				case 2: // local history
 					if(gwin->bw && gwin->bw->history)
 						ami_history_open(gwin->bw, gwin->bw->history);
@@ -650,6 +704,31 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 				case 5: // cookies tree
 					ami_open_tree(cookies_tree,AMI_TREE_COOKIES);
 				break;
+
+				case 7: // size
+					switch(subnum)
+					{
+						case 0: // decrease */
+							if(gwin->bw->scale > 0.1)
+								browser_window_set_scale(gwin->bw,
+									gwin->bw->scale - 0.1, false);
+						break;
+
+						case 1: // normal */
+							browser_window_set_scale(gwin->bw, 1.0, false);
+						break;
+
+						case 2: // increase */
+							browser_window_set_scale(gwin->bw,
+								gwin->bw->scale + 0.1, false);
+						break;
+					}
+				break;
+
+				case 8: // redraw
+					gwin->redraw_required = true;
+					gwin->new_content = true;
+				break;
 			}
 		break;
 
@@ -675,14 +754,18 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 		case 4: // settings
 			switch(itemnum)
 			{
-				case 0: // snapshot
+				case 0: // edit prefs
+					ami_gui_opts_open();
+				break;
+
+				case 2: // snapshot
 					option_window_x = gwin->win->LeftEdge;
 					option_window_y = gwin->win->TopEdge;
 					option_window_width = gwin->win->Width;
 					option_window_height = gwin->win->Height;
 				break;
 
-				case 1: // save settings
+				case 3: // save settings
 					options_write("PROGDIR:Resources/Options");
 				break;
 			}
@@ -758,4 +841,62 @@ static const ULONG ami_asl_mime_hook(struct Hook *mh,struct FileRequester *fr,st
 
 	free(mt);
 	return ret;
-} 
+}
+
+void ami_menu_update_disabled(struct gui_window *g, hlcache_handle *c)
+{
+	struct Window *win = g->shared->win;
+
+	OffMenu(win,AMI_MENU_CUT);
+	OffMenu(win,AMI_MENU_COPY);
+	OffMenu(win,AMI_MENU_PASTE);
+	OffMenu(win,AMI_MENU_CLEAR);
+
+	if(content_get_type(c) <= CONTENT_CSS)
+	{
+		OnMenu(win,AMI_MENU_SAVEAS_TEXT);
+		OnMenu(win,AMI_MENU_SAVEAS_COMPLETE);
+#ifdef WITH_PDF_EXPORT
+		OnMenu(win,AMI_MENU_SAVEAS_PDF);
+#endif
+		if(selection_defined(g->shared->bw->sel))
+		{
+			OnMenu(win,AMI_MENU_COPY);
+			OnMenu(win,AMI_MENU_CLEAR);
+
+			if(selection_read_only(g->shared->bw->sel) == false)
+				OnMenu(win,AMI_MENU_CUT);
+		}
+		if(g->c_h) OnMenu(win,AMI_MENU_PASTE);
+		OnMenu(win,AMI_MENU_SELECTALL);
+		OnMenu(win,AMI_MENU_FIND);
+		OffMenu(win,AMI_MENU_SAVEAS_IFF);
+	}
+	else
+	{
+		OffMenu(win,AMI_MENU_SAVEAS_TEXT);
+		OffMenu(win,AMI_MENU_SAVEAS_COMPLETE);
+#ifdef WITH_PDF_EXPORT
+		OffMenu(win,AMI_MENU_SAVEAS_PDF);
+#endif
+		OffMenu(win,AMI_MENU_PASTE);
+		OffMenu(win,AMI_MENU_SELECTALL);
+		OffMenu(win,AMI_MENU_CLEAR);
+		OffMenu(win,AMI_MENU_FIND);
+
+#ifdef WITH_NS_SVG
+		if(content_get_bitmap(c) || content_get_type(c) == CONTENT_SVG)
+#else
+		if(content_get_bitmap(c))
+#endif
+		{
+			OnMenu(win,AMI_MENU_COPY);
+			OnMenu(win,AMI_MENU_SAVEAS_IFF);
+		}
+		else
+		{
+			OffMenu(win,AMI_MENU_COPY);
+			OffMenu(win,AMI_MENU_SAVEAS_IFF);
+		}
+	}
+}
