@@ -32,6 +32,8 @@ enum
 	AMINS_TVWINDOW,
 	AMINS_FINDWINDOW,
 	AMINS_HISTORYWINDOW,
+	AMINS_GUIOPTSWINDOW,
+	AMINS_PRINTWINDOW,
 	AMINS_FETCHER,
 };
 
