@@ -23,6 +23,7 @@
 
 struct MinList *schedule_list;
 struct TimeRequest *tioreq;
+struct MsgPort *msgport;
 
 struct nscallback
 {
@@ -33,4 +34,5 @@ struct nscallback
 };
 
 void ami_remove_timer_event(struct nscallback *nscb);
+BOOL schedule_run(void);
 #endif
