@@ -27,8 +27,10 @@
 #include <string.h>
 #include <time.h>
 #include "content/content.h"
+#include "content/hlcache.h"
 #include "content/urldb.h"
 #include "css/css.h"
+#include "desktop/browser.h"
 #include "desktop/gui.h"
 #include "desktop/history_core.h"
 #include "desktop/plotters.h"
@@ -220,7 +222,7 @@ struct history_entry *history_clone_entry(struct history *history,
  * The page is added after the current entry and becomes current.
  */
 
-void history_add(struct history *history, struct content *content,
+void history_add(struct history *history, hlcache_handle *content,
 		char *frag_id)
 {
 	url_func_result res;
@@ -234,20 +236,21 @@ void history_add(struct history *history, struct content *content,
 
 	/* allocate space */
 	entry = malloc(sizeof *entry);
+	if (entry == NULL)
+		return;
 
-	res = url_normalize(content->url, &url);
+	res = url_normalize(content_get_url(content), &url);
 	if (res != URL_FUNC_OK) {
 		warn_user("NoMemory", 0);
+		free(entry);
 		return;
 	}
 
-	title = strdup(content->title ? content->title : url);
-
-	if (!entry || !url || !title) {
+	title = strdup(content_get_title(content));
+	if (title == NULL) {
 		warn_user("NoMemory", 0);
-		free(entry);
 		free(url);
-		free(title);
+		free(entry);
 		return;
 	}
 
@@ -302,11 +305,9 @@ void history_add(struct history *history, struct content *content,
  * \param  content  content for current entry
  */
 
-void history_update(struct history *history, struct content *content)
+void history_update(struct history *history, hlcache_handle *content)
 {
 	char *title;
-	char *url;
-	url_func_result res;
 
 	if (!history || !history->current || !history->current->bitmap)
 		return;
@@ -314,19 +315,10 @@ void history_update(struct history *history, struct content *content)
 	assert(history->current->page.url);
 	assert(history->current->page.title);
 
-	if (content->title) {
-		title = strdup(content->title);
-		if (!title) {
-			warn_user("NoMemory", 0);
-			return;
-		}
-	} else {
-		res = url_normalize(content->url, &url);
-		if (res != URL_FUNC_OK) {
-			warn_user("NoMemory", 0);
-			return;
-		}
-		title = url;
+	title = strdup(content_get_title(content));
+	if (!title) {
+		warn_user("NoMemory", 0);
+		return;
 	}
 
 	assert(title);
@@ -628,6 +620,12 @@ bool history_redraw_entry(struct history *history,
 	int tailsize = 5;
 	int xoffset = x - x0;
 	int yoffset = y - y0;
+        plot_style_t pstyle_history_rect = { 
+            .stroke_type = PLOT_OP_TYPE_SOLID,
+            .stroke_colour = c,
+            .stroke_width = entry == history->current ? 2 : 1,
+        };
+	plot_font_style_t fstyle = *plot_style_font;
 
 	if (clip) {
 		if(!plot.clip(x0 + xoffset, y0 + yoffset, x1 + xoffset, y1 + yoffset))
@@ -635,38 +633,44 @@ bool history_redraw_entry(struct history *history,
 	}
 
 	if (!plot.bitmap(entry->x + xoffset, entry->y + yoffset, WIDTH, HEIGHT,
-			entry->bitmap, 0xffffff, NULL))
+			entry->bitmap, 0xffffff, 0))
 		return false;
-	if (!plot.rectangle(entry->x - 1 + xoffset, entry->y - 1 + yoffset,
-			WIDTH + 1, HEIGHT + 1,
-			entry == history->current ? 2 : 1, c, false, false))
+	if (!plot.rectangle(entry->x - 1 + xoffset, 
+                            entry->y - 1 + yoffset,
+                            entry->x + xoffset + WIDTH, 
+                            entry->y + yoffset + HEIGHT,
+                            &pstyle_history_rect))
 		return false;
 
-	if (!nsfont.font_position_in_string(&css_base_style, entry->page.title,
+	if (!nsfont.font_position_in_string(plot_style_font, entry->page.title,
 			strlen(entry->page.title), WIDTH,
 			&char_offset, &actual_x))
 		return false;
+
+	fstyle.background = 0xffffff;
+	fstyle.foreground = c;
+
 	if (!plot.text(entry->x + xoffset, entry->y + HEIGHT + 12 + yoffset,
-			&css_base_style, entry->page.title, char_offset, 0xffffff, c))
+			entry->page.title, char_offset, &fstyle))
 		return false;
 
 	for (child = entry->forward; child; child = child->next) {
 		if (!plot.line(entry->x + WIDTH + xoffset,
 				entry->y + HEIGHT / 2 + yoffset,
 		      	entry->x + WIDTH + tailsize + xoffset,
-				entry->y + HEIGHT / 2 + yoffset, 1,
-				0x333333, false, false))
+				entry->y + HEIGHT / 2 + yoffset, 
+			       plot_style_stroke_history))
 			return false;
 		if (!plot.line(entry->x + WIDTH + tailsize + xoffset,
-				entry->y + HEIGHT / 2 + yoffset,
-				child->x - tailsize +xoffset,
-				child->y + HEIGHT / 2 + yoffset, 1,
-				0x333333, false, false))
+			       entry->y + HEIGHT / 2 + yoffset,
+			       child->x - tailsize +xoffset,
+			       child->y + HEIGHT / 2 + yoffset,
+			       plot_style_stroke_history))
 			return false;
 		if (!plot.line(child->x - tailsize + xoffset,
-				child->y + HEIGHT / 2 + yoffset,
-				child->x + xoffset, child->y + HEIGHT / 2 + yoffset, 1,
-				0x333333, false, false))
+			       child->y + HEIGHT / 2 + yoffset,
+			       child->x + xoffset, child->y + HEIGHT / 2 + yoffset,
+			       plot_style_stroke_history))
 			return false;
 		if (!history_redraw_entry(history, child, x0, y0, x1, y1, x, y, clip))
 			return false;
