@@ -36,6 +36,7 @@
 #include "content/urldb.h"
 #include "css/css.h"
 #include "desktop/options.h"
+#include "desktop/plot_style.h"
 #include "desktop/tree.h"
 #include "utils/log.h"
 #include "utils/messages.h"
@@ -50,8 +51,8 @@
 #include "beos/options.h"
 #elif defined(nsamiga)
 #include "amiga/options.h"
-#elif defined(nsfb)
-#include "framebuffer/fb_options.h"
+#elif defined(nsframebuffer)
+#include "framebuffer/options.h"
 #else
 #define EXTRA_OPTION_DEFINE
 #define EXTRA_OPTION_TABLE
@@ -103,19 +104,23 @@ bool option_animate_images = true;
 /** How many days to retain URL data for */
 int option_expire_url = 28;
 /** Default font family */
-int option_font_default = CSS_FONT_FAMILY_SANS_SERIF;
+int option_font_default = PLOT_FONT_FAMILY_SANS_SERIF;
 /** ca-bundle location */
 char *option_ca_bundle = 0;
 /** ca-path location */
 char *option_ca_path = 0;
 /** Cookie file location */
 char *option_cookie_file = 0;
-/** Cookie jar loaction */
+/** Cookie jar location */
 char *option_cookie_jar = 0;
 /** Home page location */
 char *option_homepage_url = 0;
+/** search web from url bar */
+bool option_search_url_bar = false;
 /** URL completion in url bar */
 bool option_url_suggestion = true;
+/** default web search provider */
+int option_search_provider = 0;
 /** default x position of new windows */
 int option_window_x = 0;
 /** default y position of new windows */
@@ -129,11 +134,7 @@ int option_window_screen_width = 0;
 /** height of screen when above options were saved */
 int option_window_screen_height = 0;
 /** default size of status bar vs. h scroll bar */
-#ifdef nsgtk
-int option_toolbar_status_width = 400;
-#else
 int option_toolbar_status_width = 6667;
-#endif
 /** default window scale */
 int option_scale = 100;
 /* Whether to reflow web pages while objects are fetching */
@@ -144,6 +145,7 @@ unsigned int option_min_reflow_period = 100; /* time in cs */
 #else
 unsigned int option_min_reflow_period = 25; /* time in cs */
 #endif
+bool option_core_select_menu = false;
 /** top margin of exported page*/
 int option_margin_top = DEFAULT_MARGIN_TOP_MM;
 /** bottom margin of exported page*/
@@ -229,6 +231,8 @@ struct {
 	{ "cookie_file",	OPTION_STRING,	&option_cookie_file },
 	{ "cookie_jar",		OPTION_STRING,	&option_cookie_jar },
         { "homepage_url",	OPTION_STRING,	&option_homepage_url },
+        { "search_url_bar",	OPTION_BOOL,	&option_search_url_bar},
+        { "search_provider",	OPTION_INTEGER,	&option_search_provider},
 	{ "url_suggestion",	OPTION_BOOL,	&option_url_suggestion },
 	{ "window_x",		OPTION_INTEGER,	&option_window_x },
 	{ "window_y",		OPTION_INTEGER,	&option_window_y },
@@ -243,6 +247,7 @@ struct {
 	{ "scale",		OPTION_INTEGER,	&option_scale },
 	{ "incremental_reflow",	OPTION_BOOL,	&option_incremental_reflow },
 	{ "min_reflow_period",	OPTION_INTEGER,	&option_min_reflow_period },
+ 	{ "core_select_menu",	OPTION_BOOL,	&option_core_select_menu },
 	/* Fetcher options */
 	{ "max_fetchers",	OPTION_INTEGER,	&option_max_fetchers },
 	{ "max_fetchers_per_host",
@@ -603,6 +608,10 @@ void options_load_tree_entry(xmlNode *li, struct node *directory) {
 		urldb_set_url_title(url, title);
 
 	entry = tree_create_URL_node(directory, url, data, title);
+	if (entry == NULL) {
+		/** \todo why isn't this fatal? */
+		warn_user("NoMemory", 0);
+	}
 
 	xmlFree(title);
 	free(url);
